import os
import sys
import warnings

sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(__file__))))

# Make sure messages are not translated when running the tests
# (setting the environment variable here may be too late already,
# in this case you must set it manually before running the tests).
os.environ['LANGUAGE'] = 'C'

# Enable deprecation warnings (which are disabled by default)
warnings.simplefilter('default')

try:
    try:
        import importlib.metadata as metadata
        try:
            metadata.distribution("FormEncode")
        except metadata.PackageNotFoundError as error:
            raise ImportError from error
    except ImportError:  # Python < 3.8
        import pkg_resources
        pkg_resources.require('FormEncode')
except ImportError as error:
    raise ImportError("Install FormEncode before running the tests") from error



import os
import re
import sys

from html.entities import name2codepoint

import xml.etree.ElementTree as ET

XMLParseError = ET.ParseError

base_dir = os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(
    os.path.abspath(__file__)))))
if base_dir not in sys.path:
    sys.path.insert(0, base_dir)

from formencode import htmlfill, htmlfill_schemabuilder
from formencode.doctest_xml_compare import xml_compare

import pytest


def collect_values():
    data_dir = os.path.join(os.path.dirname(__file__), 'htmlfill_data')
    for fn in os.listdir(data_dir):
        if fn.startswith('data-'):
            fn = os.path.join(data_dir, fn)
            yield fn


@pytest.mark.parametrize('filename', list(collect_values()))
def test_runfile(filename):
    f = open(filename)
    content = f.read()
    f.close()
    parts = re.split(r'---*', content)
    template = parts[0]
    expected = parts[1]
    if len(parts) == 3:
        data_content = parts[2].strip()
    elif len(parts) > 3:
        assert False, "Too many sections: %s" % parts[3:]
    else:
        data_content = ''
    namespace = {}
    if data_content:
        exec(data_content, namespace)
    data = namespace.copy()
    data['defaults'] = data.get('defaults', {})
    if 'check' in data:
        checker = data.pop('check')
    else:
        def checker(p, s):
            pass
    for name in list(data.keys()):
        if name.startswith('_') or hasattr('__builtin__', name):
            del data[name]
    listener = htmlfill_schemabuilder.SchemaBuilder()
    p = htmlfill.FillingParser(listener=listener, **data)
    p.feed(template)
    p.close()
    output = p.text()

    def reporter(v):
        print(v)

    try:
        output_xml = ET.XML(output)
        expected_xml = ET.XML(expected)
    except XMLParseError:
        comp = output.strip() == expected.strip()
    else:
        comp = xml_compare(output_xml, expected_xml, reporter)
    if not comp:
        print('---- Output:   ----')
        print(output)
        print('---- Expected: ----')
        print(expected)
        assert False
    checker(p, listener.schema())
    checker(p, htmlfill_schemabuilder.parse_schema(template))


def test_no_trailing_newline():
    assert (htmlfill.render('<html><body></body></html>', {}, {})
            == '<html><body></body></html>')


def test_escape_defaults():
    rarr = chr(name2codepoint['rarr'])
    assert (htmlfill.render('<input type="submit" value="next&gt;&rarr;">', {}, {})
            == '<input type="submit" value="next&gt;%s">' % rarr)
    assert (htmlfill.render('<input type="submit" value="1&amp;2">', {}, {})
            == '<input type="submit" value="1&amp;2">')
    assert (htmlfill.render('<input type="submit" value="Japan - &#x65E5;&#x672C; Nihon" />',
                            {}, {}) ==
            '<input type="submit" value="Japan - 日本 Nihon" />')


def test_xhtml():
    result = htmlfill.render('<form:error name="code"/>', errors={'code': 'an error'})
    assert 'an error' in result


def test_html5():
    result = htmlfill.render('<input type="number" name="quantity">', {'quantity': '10'})
    assert result == '<input type="number" name="quantity" value="10">'
    try:
        result = htmlfill.render('<input type="unknown" name="quantity">', {'quantity': '10'})
    except AssertionError as e:
        assert "I don't know about this kind of <input>: unknown at 1:0" in str(e)
    result = htmlfill.render('<input type="unknown" name="quantity">', {'quantity': '10'}, text_as_default=True)
    assert result == '<input type="unknown" name="quantity" value="10">'


def test_trailing_error():
    assert (htmlfill.render('<input type="text" name="email">', errors={'email': 'error'},
                            prefix_error=False)
            == '<input type="text" name="email" class="error" value=""><!-- for: email -->\n<span class="error-message">error</span><br />\n')
    assert (htmlfill.render('<textarea name="content"></textarea>', errors={'content': 'error'},
                            prefix_error=False)
            == '<textarea name="content" class="error"></textarea><!-- for: content -->\n<span class="error-message">error</span><br />\n')
    assert (htmlfill.render('<select name="type"><option value="foo">foo</option></select>', errors={'type': 'error'},
                            prefix_error=False)
            == '<select name="type" class="error"><option value="foo">foo</option></select><!-- for: type -->\n<span class="error-message">error</span><br />\n')


def test_iferror():
    assert (htmlfill.render('<form:iferror name="field1">an error</form:iferror>', errors={}, auto_insert_errors=False)
            == '')
    assert (htmlfill.render('<form:iferror name="field1">an error</form:iferror>', errors={'field1': 'foo'}, auto_insert_errors=False)
            == 'an error')
    assert (htmlfill.render('<form:iferror name="not field1">no errors</form:iferror>', errors={}, auto_insert_errors=False)
            == 'no errors')
    assert (htmlfill.render('<form:iferror name="not field1">no errors</form:iferror>', errors={'field1': 'foo'}, auto_insert_errors=False)
            == '')
    assert (htmlfill.render('<form:iferror name="field1">errors</form:iferror><form:iferror name="not field1">no errors</form:iferror>',
                            errors={}, auto_insert_errors=False)
            == 'no errors')
    assert (htmlfill.render('<form:iferror name="field1">errors</form:iferror><form:iferror name="not field1">no errors</form:iferror>',
                            errors={'field1': 'foo'}, auto_insert_errors=False)
            == 'errors')
    try:
        htmlfill.render('<form:iferror noname="nothing">errors</form:iferror>')
    except AssertionError as e:
        assert str(e) == "Name attribute in <iferror> required at 1:0"


def test_literal():
    assert (htmlfill.render('<form:error name="foo" />',
                            errors={'foo': htmlfill.htmlliteral('<test>')})
            == '<span class="error-message"><test></span><br />\n')


def test_image_submit():
    assert (htmlfill.render('<input name="image-submit" type="image" src="foo.jpg" value="bar">',
                            defaults={'image-submit': 'blahblah'})
            == '<input name="image-submit" type="image" src="foo.jpg" value="bar">')


def test_checkbox():
    assert (htmlfill.render('<input name="checkbox" type="checkbox" value="bar">',
                            defaults={'checkbox': 'bar'})
            == '<input name="checkbox" type="checkbox" value="bar" checked="checked">')
    assert (htmlfill.render('<input name="checkbox" type="checkbox">',
                            defaults={'checkbox': ''})
            == '<input name="checkbox" type="checkbox">')
    assert (htmlfill.render('<input name="checkbox" type="checkbox">',
                            defaults={'checkbox': ''}, checkbox_checked_if_present=True)
            == '<input name="checkbox" type="checkbox" checked="checked">')
    assert (htmlfill.render('<input name="checkbox" type="checkbox" value="bar">',
                            defaults={'checkbox': ''}, checkbox_checked_if_present=True)
            == '<input name="checkbox" type="checkbox" value="bar">')
    assert (htmlfill.render('<input name="checkbox" type="checkbox" value="">',
                            defaults={'checkbox': ''}, checkbox_checked_if_present=True)
            == '<input name="checkbox" type="checkbox" value="" checked="checked">')
    assert (htmlfill.render('<input name="checkbox" type="checkbox" value="">',
                            defaults={'checkbox': ''})
            == '<input name="checkbox" type="checkbox" value="">')


def test_unicode():
    assert (htmlfill.render('<input type="checkbox" name="tags" value="2" />',
                           dict(tags=[])) ==
            '<input type="checkbox" name="tags" value="2" />')


def test_password():
    assert (htmlfill.render('<input name="password" type="password" value="">',
                            defaults={'password': 'secure passwd'})
            == '<input name="password" type="password" value="secure passwd">')
    assert (htmlfill.render('<input name="password" type="password" value="">',
                            defaults={'password': 'secure passwd'},
                            skip_passwords=True)
            == '<input name="password" type="password" value="">')
    assert (htmlfill.render('<input name="password" type="password">',
                            defaults={'password': 'secure passwd'})
            == '<input name="password" type="password" value="secure passwd">')
    assert (htmlfill.render('<input name="password" type="password">',
                            defaults={'password': 'secure passwd'},
                            skip_passwords=True)
            == '<input name="password" type="password">')


def test_not_force_defaults_text():
    html = """<input type="text" name="text-1" class="my_text" value="i like this text" />"""
    rendered_html = htmlfill.render(html, defaults=dict(),
                                    force_defaults=False)
    assert html == rendered_html, rendered_html


def test_not_force_defaults_text_value():
    html = """<input type="text" name="text-1" class="my_text" value="i like this text" />"""
    expected_html = """<input type="text" name="text-1" class="my_text" value="this text is better" />"""
    rendered_html = htmlfill.render(html, defaults={"text-1": "this text is better"},
                                    force_defaults=False)
    assert expected_html == rendered_html, rendered_html


def test_not_force_defaults_text_explicit_empty_value():
    html = """<input type="text" name="text-1" class="my_text" value="i like this text" />"""
    expected_html = """<input type="text" name="text-1" class="my_text" value="" />"""
    rendered_html = htmlfill.render(html, defaults={"text-1": ""},
                                    force_defaults=False)
    assert expected_html == rendered_html, rendered_html


def test_force_defaults_text():
    html = """<input type="text" name="text-1" class="my_text" value="i like this text" />"""
    expected_html = """<input type="text" name="text-1" class="my_text" value="" />"""
    rendered_html = htmlfill.render(html, defaults=dict())
    assert expected_html == rendered_html, rendered_html


def test_not_force_defaults_textarea():
    html = """<textarea name="textarea-1" class="my_textarea">i like this text</textarea>"""
    rendered_html = htmlfill.render(html, defaults=dict(),
                                    force_defaults=False)
    assert html == rendered_html, rendered_html


def test_not_force_defaults_textarea_value():
    html = """<textarea name="textarea-1" class="my_textarea">i like this text</textarea>"""
    expected_html = """<textarea name="textarea-1" class="my_textarea">this text is better</textarea>"""
    rendered_html = htmlfill.render(html, defaults={"textarea-1": "this text is better"},
                                    force_defaults=False)
    assert expected_html == rendered_html, rendered_html


def test_force_defaults_textarea():
    html = """<textarea name="textarea-1" class="my_textarea">i like this text</textarea>"""
    expected_html = \
        """<textarea name="textarea-1" class="my_textarea"></textarea>"""
    rendered_html = htmlfill.render(html, defaults=dict())
    assert expected_html == rendered_html, rendered_html


def test_not_force_defaults_password():
    html = """<input type="password" name="password-1" class="my_password" value="i like this password" />"""
    rendered_html = htmlfill.render(html, defaults=dict(),
                                    force_defaults=False)
    assert html == rendered_html, rendered_html


def test_not_force_defaults_password_value():
    html = """<input type="password" name="password-1" class="my_password" value="i like this password" />"""
    expected_html = """<input type="password" name="password-1" class="my_password" value="this password is better" />"""
    rendered_html = htmlfill.render(html, defaults={"password-1": "this password is better"},
                                    force_defaults=False)
    assert expected_html == rendered_html, rendered_html


def test_not_force_defaults_password_explicit_empty_value():
    html = """<input type="password" name="password-1" class="my_password" value="i like this password" />"""
    expected_html = """<input type="password" name="password-1" class="my_password" value="" />"""
    rendered_html = htmlfill.render(html, defaults={"password-1": ""},
                                    force_defaults=False)
    assert expected_html == rendered_html, rendered_html


def test_force_defaults_password():
    html = """<input type="password" name="password-1" class="my_password" value="i like this password" />"""
    expected_html = """<input type="password" name="password-1" class="my_password" value="" />"""
    rendered_html = htmlfill.render(html, defaults=dict())
    assert expected_html == rendered_html, rendered_html


def test_not_force_defaults_checkbox():
    html = """<input type="checkbox" name="checkbox-1" class="my_checkbox" checked="checked" value="cb">"""
    rendered_html = htmlfill.render(html, defaults=dict(),
                                    force_defaults=False)
    assert html == rendered_html, rendered_html


def test_force_defaults_checkbox():
    html = """<input type="checkbox" name="checkbox-1" class="my_checkbox" checked="checked" value="cb">"""
    expected_html = \
        """<input type="checkbox" name="checkbox-1" class="my_checkbox" value="cb">"""
    rendered_html = htmlfill.render(html, defaults=dict())
    assert expected_html == rendered_html, rendered_html


def test_not_force_defaults_checkbox_default_unchecked():
    html = """<input type="checkbox" name="checkbox-1" class="my_checkbox" checked="checked" value="cb">"""
    expected_html = \
        """<input type="checkbox" name="checkbox-1" class="my_checkbox" value="cb">"""
    rendered_html = htmlfill.render(html, defaults={"checkbox-1": False})
    assert expected_html == rendered_html, rendered_html


def test_not_force_defaults_checkbox_default_checked():
    html = """<input type="checkbox" name="checkbox-1" class="my_checkbox" value="cb">"""
    expected_html = \
        """<input type="checkbox" name="checkbox-1" class="my_checkbox" value="cb" checked="checked">"""
    rendered_html = htmlfill.render(html, defaults={"checkbox-1": "cb"},
                                    force_defaults=False)
    assert expected_html == rendered_html, rendered_html


def test_not_force_defaults_radio():
    html = """<input type="radio" name="radio-1" class="my_radio" checked="checked" value="cb">"""
    rendered_html = htmlfill.render(html, defaults=dict(),
                                    force_defaults=False)
    assert html == rendered_html, rendered_html


def test_not_force_defaults_radio_unchecked():
    html = """<input type="radio" name="radio-1" class="my_radio" checked="checked" value="cb">"""
    expected_html = """<input type="radio" name="radio-1" class="my_radio" value="cb">"""
    rendered_html = htmlfill.render(html, defaults={"radio-1": "ba"},
                                    force_defaults=False)
    assert expected_html == rendered_html, rendered_html


def test_not_force_defaults_radio_checked():
    html = """<input type="radio" name="radio-1" class="my_radio" value="cb">"""
    expected_html = """<input type="radio" name="radio-1" class="my_radio" value="cb" checked="checked">"""
    rendered_html = htmlfill.render(html, defaults={"radio-1": "cb"},
                                    force_defaults=False)
    assert expected_html == rendered_html, rendered_html


def test_force_defaults_radio():
    html = """<input type="radio" name="radio-1" class="my_radio" checked="checked" value="cb">"""
    expected_html = """<input type="radio" name="radio-1" class="my_radio" value="cb">"""
    rendered_html = htmlfill.render(html, defaults=dict())
    assert expected_html == rendered_html, rendered_html


def test_not_force_defaults_select():
    html = """
<select name="select-1" class="my_select">
  <option value="option-1" selected="selected">this is option-1</option>
</select>
"""
    rendered_html = htmlfill.render(html, defaults=dict(),
                                    force_defaults=False)
    assert html == rendered_html, rendered_html


def test_not_force_defaults_select_selected():
    html = """
<select name="select-1" class="my_select">
  <option value="option-1">this is option-1</option>
</select>
"""
    expected_html = """
<select name="select-1" class="my_select">
  <option value="option-1" selected="selected">this is option-1</option>
</select>
"""
    rendered_html = htmlfill.render(html, defaults={"select-1": "option-1"},
                                    force_defaults=False)
    assert expected_html == rendered_html, rendered_html


def test_not_force_defaults_select_not_selected():
    html = """
<select name="select-1" class="my_select">
  <option value="option-1" selected="selected">this is option-1</option>
</select>
"""
    expected_html = """
<select name="select-1" class="my_select">
  <option value="option-1">this is option-1</option>
</select>
"""
    rendered_html = htmlfill.render(html, defaults={"select-1": "option-2"},
                                    force_defaults=False)
    assert expected_html == rendered_html, rendered_html


def test_force_defaults_select():
    html = """
<select name="select-1" class="my_select">
  <option value="option-1" selected="selected">this is option-1</option>
</select>
"""
    expected_html = """
<select name="select-1" class="my_select">
  <option value="option-1">this is option-1</option>
</select>
"""
    rendered_html = htmlfill.render(html, defaults=dict())
    assert expected_html == rendered_html, rendered_html


def test_select_empty_option_value_selected():
    html = """
<select name="select-1" class="my_select">
  <option value="">this is option-1</option>
</select>
"""
    expected_html = """
<select name="select-1" class="my_select">
  <option value="" selected="selected">this is option-1</option>
</select>
"""
    rendered_html = htmlfill.render(html, defaults={"select-1": ""})
    assert expected_html == rendered_html, rendered_html


def test_select_empty_option_value_not_selected():
    html = """
<select name="select-1" class="my_select">
  <option value="">this is option-1</option>
</select>
"""
    expected_html = """
<select name="select-1" class="my_select">
  <option value="">this is option-1</option>
</select>
"""
    rendered_html = htmlfill.render(html, defaults={})
    assert expected_html == rendered_html, rendered_html


def test_script_quoting():
    html = """
<script>Some <weird JS</script>
Then a form <input type="text" name="name">
"""
    expected_html = """
<script>Some <weird JS</script>
Then a form <input type="text" name="name" value="foo">
"""
    rendered_html = htmlfill.render(html, defaults=dict(name="foo"))
    assert expected_html == rendered_html, rendered_html


def test_error_class_textarea():
    assert (htmlfill.render('<textarea name="content"></textarea>', errors={'content': 'error'})
            == '<!-- for: content -->\n<span class="error-message">error</span><br />\n<textarea name="content" class="error"></textarea>')


def test_mix_str_and_unicode():
    html = '<input type="text" name="cheese">'
    uhtml = str(html)
    cheese = dict(cheese='Käse')
    ucheese = dict(cheese='Käse')
    expected = '<input type="text" name="cheese" value="Käse">'
    rendered = htmlfill.render(html, defaults=cheese, encoding='utf-8')
    assert expected == rendered
    rendered = htmlfill.render(html, defaults=ucheese, encoding='utf-8')
    assert expected == rendered
    rendered = htmlfill.render(uhtml, defaults=cheese, encoding='utf-8')
    assert expected == rendered
    rendered = htmlfill.render(uhtml, defaults=ucheese, encoding='utf-8')
    assert expected == rendered



import unittest

from formencode import compound, Invalid
from formencode.validators import DictConverter


class TestCompoundValidator(unittest.TestCase):

    def setUp(self):
        self.validator = compound.CompoundValidator()

    def test_repr(self):
        r = repr(self.validator)
        self.assertNotIn('validatorArgs', r)
        self.assertIn('validators=[]', r)

    def test_to_python(self):
        self.assertRaises(NotImplementedError,
            self.validator.to_python, 1)

    def test_from_python(self):
        self.assertRaises(NotImplementedError,
            self.validator.from_python, 1)

    def test_clone(self):
        clone = self.validator()
        self.assertEqual(type(clone), type(self.validator))


class TestAllCompoundValidator(unittest.TestCase):

    def setUp(self):
        self.validator = compound.All(
            validators=[DictConverter({2: 1}), DictConverter({3: 2})])

    def test_repr(self):
        r = repr(self.validator)
        self.assertNotIn('validatorArgs', r)
        self.assertEqual(r.count('DictConverter'), 2)

    def test_to_python(self):
        self.assertEqual(self.validator.to_python(3), 1)

    def test_from_python(self):
        self.assertEqual(self.validator.from_python(1), 3)

    def test_clone(self):
        clone = self.validator()
        self.assertEqual(clone.to_python(3), 1)


class TestAnyCompoundValidator(unittest.TestCase):

    def setUp(self):
        self.validator = compound.Any(
            validators=[DictConverter({2: 'c'}), DictConverter({2: 'b'}),
                DictConverter({1: 'b'})])

    def test_repr(self):
        r = repr(self.validator)
        self.assertNotIn('validatorArgs', r)
        self.assertEqual(r.count('DictConverter'), 3)

    def test_to_python(self):
        # Should stop before 'c' coming from the right.
        self.assertEqual(self.validator.to_python(2), 'b')

    def test_from_python(self):
        # Should stop before 1 coming from the left.
        self.assertEqual(self.validator.from_python('b'), 2)

    def test_to_python_error(self):
        try:
            self.validator.to_python(3)
        except Invalid as e:
            self.assertIn('Enter a value from: 2', str(e))
        else:
            self.fail('Invalid should be raised when no validator succeeds.')

    def test_clone(self):
        clone = self.validator()
        self.assertEqual(clone.to_python(2), 'b')


class TestPipeCompoundValidator(unittest.TestCase):

    def setUp(self):
        self.validator = compound.Pipe(
            validators=[DictConverter({1: 2}), DictConverter({2: 3})])

    def test_repr(self):
        r = repr(self.validator)
        self.assertNotIn('validatorArgs', r)
        self.assertEqual(r.count('DictConverter'), 2)

    def test_to_python(self):
        self.assertEqual(self.validator.to_python(1), 3)

    def test_from_python(self):
        self.assertEqual(self.validator.from_python(3), 1)

    def test_clone(self):
        clone = self.validator()
        self.assertEqual(clone.to_python(1), 3)



import doctest

from formencode.htmlgen import html

# A test value that can't be encoded as ascii:
uni_value = '\xff'


def test_basic():
    output = '<a href="test">hey there</a>'
    assert str(html.a(href='test')('hey there')) == output
    assert str(html.a('hey there')(href='test')) == output
    assert str(html.a(href='test', c='hey there')) == output
    assert str(html.a('hey there', href='test')) == output
    assert str(html.a(href='test')('hey ', 'there')) == output
    assert str(html.a(href='test')(['hey ', 'there'])) == output


def test_compound():
    output = '<b>Hey <i>you</i>!</b>'
    assert str(html.b('Hey ', html.i('you'), '!')) == output
    assert str(html.b()('Hey ')(html.i()('you'))('!')) == output
    inner = html('Hey ', html.i('you'), '!')
    assert html.str(inner) == 'Hey <i>you</i>!'
    assert str(inner) == 'Hey <i>you</i>!'
    assert str(html.b(inner)) == output


def test_unicode():
    try:
        uni_value.encode('ascii')
    except ValueError:
        pass
    else:
        assert False, (
            "We need something that can't be ASCII-encoded: %r (%r)"
            % (uni_value, uni_value.encode('ascii')))
    assert str(html.b(uni_value)) == '<b>%s</b>' % uni_value


def test_quote():
    assert html.quote('<hey>!') == '&lt;hey&gt;!'
    assert html.quote(uni_value) == uni_value
    assert html.quote(None) == ''
    assert html.str(None) == ''
    assert str(html.b('<hey>')) == '<b>&lt;hey&gt;</b>'


def test_comment():

    def strip(s):
        """ElementTree in Py < 2.7 adds whitespace, strip this."""
        s = str(s).strip()
        if s.startswith('<!--') and s.endswith('-->'):
            s = '<!--%s-->' % s[4:-3].strip()
        return s

    assert strip(html.comment('test')) == '<!--test-->'
    assert strip(html.comment(uni_value)) == '<!--%s-->' % uni_value
    assert strip(html.comment('test')('this')) == '<!--testthis-->'


def test_none():
    assert html.str(None) == ''
    assert str(html.b(class_=None)('hey')) == '<b>hey</b>'
    assert str(html.b(class_=' ')(None)) == '<b class=" " />'


def test_namespace():
    output = '<b tal:content="options/whatever" />'
    assert str(html.b(**{'tal:content': 'options/whatever'})) == output
    assert str(html.b(tal__content='options/whatever')) == output


if __name__ == '__main__':
    # It's like a super-mini py.test...
    for name, value in globals().items():
        if name.startswith('test'):
            print(name)
            value()
    from formencode import htmlgen
    doctest.testmod(htmlgen)
    print('doctest')



import os
import sys
import doctest

from formencode import compound
from formencode import htmlfill
from formencode import htmlgen
from formencode import national
from formencode import schema
from formencode import validators

import pytest


"""Modules that will have their doctests tested."""
modules = [compound, htmlfill, htmlgen, national, schema, validators]


"""Text files that will have their doctests tested."""
text_files = [
    "docs/htmlfill.txt",
    "docs/Validator.txt",
    "tests/non_empty.txt",
]


"""Used to resolve text files to absolute paths."""
base = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


def doctest_file(document, verbose, raise_error):
    failure_count, test_count = doctest.testfile(
        document,
        module_relative=False,
        optionflags=doctest.ELLIPSIS | doctest.IGNORE_EXCEPTION_DETAIL,
        verbose=verbose,
    )
    if raise_error:
        assert test_count > 0
        assert failure_count == 0


def doctest_module(document, verbose, raise_error):
    failure_count, test_count = doctest.testmod(
        document,
        optionflags=doctest.ELLIPSIS | doctest.IGNORE_EXCEPTION_DETAIL,
        verbose=verbose,
    )
    if raise_error:
        assert test_count > 0
        assert failure_count == 0


def set_func_description(fn, description):
    """Wrap function and set description attr for pytest to display."""

    def _wrapper(*a_test_args):
        fn(*a_test_args)

    _wrapper.description = description
    return _wrapper


def collect_functions():
    verbose = False
    raise_error = True
    for document in text_files + modules:
        if isinstance(document, str):
            name = "Doctests for %s" % (document,)
            if not document.startswith(os.sep):
                document = os.path.join(base, document)
            yield set_func_description(
                doctest_file, name
            ), document, verbose, raise_error
        else:
            name = "Doctests for %s" % (document.__name__,)
            yield set_func_description(
                doctest_module, name
            ), document, verbose, raise_error


@pytest.mark.parametrize(
    "testfn,document,verbose,raise_error", list(collect_functions())
)
def test_doctests(testfn, document, verbose, raise_error):
    """Generate each doctest."""
    testfn(document, verbose, raise_error)


if __name__ == "__main__":
    # Call this file directly if you want to test doctests.
    args = sys.argv[1:]
    verbose = False
    if "-v" in args:
        args.remove("-v")
        verbose = True
    if not args:
        args = text_files + modules
    raise_error = False
    for fn in args:
        if isinstance(fn, str):
            fn = os.path.join(base, fn)
            doctest_file(fn, verbose, raise_error)
        else:
            doctest_module(fn, verbose, raise_error)



from formencode import htmlfill


# ==============================================================================


def test_defaults_legacy():
    html = """
<input type="text" name="foo" value="bar" />
<input type="text" name="foo" value="biz" />
<input type="text" name="foo" value="bash" />
"""
    expected_html = """
<input type="text" name="foo" value="bang" />
<input type="text" name="foo" value="bang" />
<input type="text" name="foo" value="bang" />
"""
    rendered_html = htmlfill.render(html, defaults={"foo": "bang"},
                                    force_defaults=True)
    assert expected_html == rendered_html


def test_defaults_attr_ignore():
    html = """
<input type="text" name="foo" value="bar" data-formencode-ignore="1" />
<input type="text" name="foo" value="" />
<input type="text" name="foo" value="bash" data-formencode-ignore="1" />
<input type="text" name="foo" value="bash" data-formencode-ignore="" />
<input type="text" name="foo" value="bash" data-formencode-ignore />
"""
    expected_html = """
<input type="text" name="foo" value="bar" data-formencode-ignore="1" />
<input type="text" name="foo" value="bang" />
<input type="text" name="foo" value="bash" data-formencode-ignore="1" />
<input type="text" name="foo" value="bash" data-formencode-ignore="" />
<input type="text" name="foo" value="bash" data-formencode-ignore />
"""
    rendered_html = htmlfill.render(html, defaults={"foo": "bang"},
                                    force_defaults=True,
                                    data_formencode_ignore=True)
    assert expected_html == rendered_html


def test_defaults_attr_form():
    html = """
<input type="text" name="foo" value="bar" data-formencode-form="a" />
<input type="text" name="foo" value="" data-formencode-form="b" />
<input type="text" name="foo" value="bash" data-formencode-form="c" />
"""
    expected_html = """
<input type="text" name="foo" value="bar" data-formencode-form="a" />
<input type="text" name="foo" value="bang" data-formencode-form="b" />
<input type="text" name="foo" value="bash" data-formencode-form="c" />
"""
    rendered_html = htmlfill.render(html, defaults={"foo": "bang"},
                                    force_defaults=True,
                                    data_formencode_form="b",)
    assert expected_html == rendered_html


# ==============================================================================

def test_error_legacy():
    html = """
<input type="text" name="foo" value="bar" />
<input type="text" name="foo" value="biz" />
<input type="text" name="foo" value="bash" />
"""
    expected_html = """
<!-- for: foo -->
<span class="error-message">bang</span><br />
<input type="text" name="foo" value="" class="error" />
<input type="text" name="foo" value="" class="error" />
<input type="text" name="foo" value="" class="error" />
"""
    rendered_html = htmlfill.render(html, errors={"foo": "bang"},
                                    prefix_error=True)
    assert expected_html == rendered_html


def test_error_attr_ignore():
    html = """
<input type="text" name="foo" value="bar" data-formencode-form="a" />
<input type="text" name="foo" value="biz" data-formencode-form="b" />
<input type="text" name="foo" value="bash" data-formencode-form="c" />
"""
    expected_html = """
<input type="text" name="foo" value="bar" data-formencode-form="a" />
<!-- for: foo -->
<span class="error-message">bang</span><br />
<input type="text" name="foo" value="" data-formencode-form="b" class="error" />
<input type="text" name="foo" value="bash" data-formencode-form="c" />
"""
    rendered_html = htmlfill.render(html, errors={"foo": "bang"},
                                    force_defaults=True,
                                    data_formencode_form="b",)
    assert expected_html == rendered_html


def test_error_attr_form():
    html = """
<input type="text" name="foo" value="bar" data-formencode-form="a" />
<input type="text" name="foo" value="" data-formencode-form="b" />
<input type="text" name="foo" value="bash" data-formencode-form="c" />
"""
    expected_html = """
<input type="text" name="foo" value="bar" data-formencode-form="a" />
<!-- for: foo -->
<span class="error-message">bang</span><br />
<input type="text" name="foo" value="" data-formencode-form="b" class="error" />
<input type="text" name="foo" value="bash" data-formencode-form="c" />
"""
    rendered_html = htmlfill.render(html, errors={"foo": "bang"},
                                    force_defaults=True,
                                    data_formencode_form="b",)
    assert expected_html == rendered_html


def test_error_attr_form_alt():
    """note that formencode doesn't keep an indent on the replacement
        mixes concepts
        note a few things:
            1. we expect a leading "<!-- for: apple -->" block, because we are ignoring that tag
            1. we expect the leading "<!-- for: apple -->" block to not have an initial newline (\n)
    """
    html = """
<form data-formencode-form="a">
    <input type="text" name="bar" value="foo" data-formencode-form="a" />
    <input type="text" name="foo" value="bar" data-formencode-form="a" />
</form>
<form data-formencode-form="b">
    <input type="text" name="bar" value="foo" data-formencode-form="b" />
    <input type="text" name="foo" value="" data-formencode-form="b" />
</form>
<form data-formencode-form="c">
    <input type="text" name="bar" value="foo" data-formencode-form="c" />
    <input type="text" name="foo" value="bash" data-formencode-form="c" />
    <input type="text" name="apple" value="pear" data-formencode-form="c" data-formencode-ignore="1" />
</form>
"""
    expected_html = """<!-- for: apple -->
<span class="error-message">orange</span><br />

<form data-formencode-form="a">
    <input type="text" name="bar" value="foo" data-formencode-form="a" />
    <input type="text" name="foo" value="bar" data-formencode-form="a" />
</form>
<form data-formencode-form="b">
    <input type="text" name="bar" value="foo" data-formencode-form="b" />
    <input type="text" name="foo" value="" data-formencode-form="b" />
</form>
<form data-formencode-form="c">
    <input type="text" name="bar" value="bang" data-formencode-form="c" />
    <!-- for: foo -->
<span class="error-message">bang</span><br />
<input type="text" name="foo" value="" data-formencode-form="c" class="error" />
    <input type="text" name="apple" value="pear" data-formencode-form="c" data-formencode-ignore="1" />
</form>
"""
    rendered_html = htmlfill.render(html, defaults={"bar": "bang"},
                                    errors={"foo": "bang", "apple": "orange"},
                                    force_defaults=True,
                                    data_formencode_form="c",
                                    data_formencode_ignore=True,
                                    )
    assert expected_html == rendered_html


if __name__ == '__main__':
    test_defaults_legacy()
    test_defaults_attr_ignore()
    test_defaults_attr_form()

    test_error_legacy()
    test_error_attr_ignore()
    test_error_attr_form()
    test_error_attr_form_alt()



import unittest

from formencode.variabledecode import variable_decode, variable_encode


class TestVariableDecode(unittest.TestCase):

    def test_list_decode(self):
        src = {'a-0': 'a', 'a-1': 'b', 'a-2': 'c'}
        expect = {'a': ['a', 'b', 'c']}

        self.assertEqual(expect, variable_decode(src))

    def test_list_decode_non_int(self):
        src = {'a-0': 'a', 'a-a': 'b', 'a-2': 'c'}
        expect = {'a': ['a', 'c'], 'a-a': 'b'}

        self.assertEqual(expect, variable_decode(src))

    def test_list_decode_double_dash(self):
        src = {'a-0': 'a', 'a-1-2': 'b', 'a-3': 'c'}
        expect = {'a': ['a', 'c'], 'a-1-2': 'b'}

        self.assertEqual(expect, variable_decode(src))

    def test_list_decode_non_int_nested(self):
        src = {'a-0.name': 'a', 'a-a.name': 'b', 'a-2.name': 'c'}
        expect = {'a': [{'name': 'a'}, {'name': 'c'}], 'a-a': {'name': 'b'}}

        self.assertEqual(expect, variable_decode(src))

    def test_dict_decode(self):
        src = {'a.a': 'a', 'a.b': 'b', 'a.c': 'c'}
        expect = {'a': {'a': 'a', 'b': 'b', 'c': 'c'}}

        self.assertEqual(expect, variable_decode(src))

    def test_list_dict(self):
        src = {'a-0.name': 'a', 'a-1.name': 'b', 'a-2.name': 'c'}
        expect = {'a': [{'name': 'a'}, {'name': 'b'}, {'name': 'c'}]}

        self.assertEqual(expect, variable_decode(src))

    def test_dict_list_dict(self):
        src = {'a.b-0.name': 'a', 'a.b-1.name': 'b', 'a.b-2.name': 'c'}
        expect = {'a': {'b': [{'name': 'a'}, {'name': 'b'}, {'name': 'c'}]}}

        self.assertEqual(expect, variable_decode(src))

    def test_list_dict_bad_key(self):
        src = {'a-0.good': 'a', 'a.0.bad': 'b', 'a-1.good': 'c'}
        expect = {'a': [{'good': 'a'}, {'good': 'c'}, {'bad': 'b'}]}

        self.assertEqual(expect, variable_decode(src))


class TestVariableEncode(unittest.TestCase):

    def test_list_encode(self):
        src = {'a': ['a', 'b', 'c']}
        expect = {'a--repetitions': '3', 'a-0': 'a', 'a-1': 'b', 'a-2': 'c'}

        self.assertEqual(expect, variable_encode(src))

    def test_list_encode_non_int(self):
        src = {'a': ['a', 'c'], 'a-a': 'b'}
        expect = {'a--repetitions': '2', 'a-0': 'a', 'a-a': 'b', 'a-1': 'c'}

        self.assertEqual(expect, variable_encode(src))

    def test_dict_encode(self):
        src = {'a': {'a': 'a', 'b': 'b', 'c': 'c'}}
        expect = {'a.a': 'a', 'a.b': 'b', 'a.c': 'c'}

        self.assertEqual(expect, variable_encode(src))



import datetime
import unittest
import pytest

from formencode import validators
from formencode.validators import Invalid
from formencode.variabledecode import NestedVariables
from formencode.schema import Schema
from formencode.foreach import ForEach
from formencode.api import NoDefault


def validate(validator, value):
    try:
        return validator.to_python(value)
    except Invalid as e:
        return e.unpack_errors()


def validate_from(validator, value):
    try:
        return validator.from_python(value)
    except Invalid as e:
        return e.unpack_errors()


class TestValidators(unittest.TestCase):

    def testHelp(self):
        from pydoc import text, plain
        s = plain(text.document(validators))
        self.assertIn('Validator/Converters for use with FormEncode.', s)
        self.assertIn('class Bool', s)
        self.assertIn('Always Valid, returns True or False', s)
        self.assertIn('class Email', s)
        self.assertIn('Validate an email address.', s)
        self.assertIn('class FieldsMatch', s)
        self.assertIn('Tests that the given fields match', s)


class TestByteStringValidator(unittest.TestCase):

    def setUp(self):
        self.validator = validators.ByteString
        self.messages = self.validator.message

    def test_alias(self):
        self.assertIsNot(self.validator, validators.String)

    def test_docstring(self):
        doc = self.validator.__doc__
        self.assertIn(
            'Enter a value not more than ``%(max)i`` characters long', doc)

    def test_sv_min(self):
        sv = self.validator(min=2, accept_python=False)
        self.assertEqual(sv.to_python("foo"), "foo")
        self.assertEqual(validate(sv, "x"),
            self.messages('tooShort', None, min=2))
        self.assertEqual(validate(sv, None), self.messages('empty', None))
        # should be completely invalid?
        self.assertEqual(validate(sv, []), self.messages('empty', None, min=2))
        self.assertEqual(sv.from_python(['x', 'y']), 'x, y')

    def test_sv_not_empty(self):
        sv = self.validator(not_empty=True)
        self.assertEqual(validate(sv, ""), self.messages('empty', None))
        self.assertEqual(validate(sv, None), self.messages('empty', None))
        # should be completely invalid?
        self.assertEqual(validate(sv, []), self.messages('empty', None))
        self.assertEqual(validate(sv, {}), self.messages('empty', None))

    def test_sv_string_conversion(self):
        sv = self.validator(not_empty=False)
        self.assertEqual(sv.from_python(2), "2")
        self.assertEqual(sv.from_python([]), "")


class TestUnicodeStringValidator(unittest.TestCase):

    def setUp(self):
        self.validator = validators.UnicodeString

    def test_alias(self):
        self.assertIs(self.validator, validators.String)

    def test_docstring(self):
        doc = self.validator.__doc__
        self.assertIn('Invalid data or incorrect encoding', doc)

    def test_unicode(self):
        un = self.validator()
        self.assertEqual(un.to_python(12), '12')
        self.assertIs(type(un.to_python(12)), str)
        self.assertEqual(un.from_python(12), '12'.encode('ascii'))
        self.assertIs(type(un.from_python(12)), bytes)

    def test_unicode_encoding(self):
        uv = self.validator()
        us = 'käse'
        u7s, u8s = us.encode('utf-7'), us.encode('utf-8')
        self.assertEqual(uv.to_python(u8s), us)
        self.assertIs(type(uv.to_python(u8s)), str)
        self.assertEqual(uv.from_python(us), u8s)
        self.assertIs(type(uv.from_python(us)), bytes)
        uv = self.validator(encoding='utf-7')
        self.assertEqual(uv.to_python(u7s), us)
        self.assertIs(type(uv.to_python(u7s)), str)
        self.assertEqual(uv.from_python(us), u7s)
        self.assertIs(type(uv.from_python(us)), bytes)
        uv = self.validator(inputEncoding='utf-7')
        self.assertEqual(uv.to_python(u7s), us)
        self.assertIs(type(uv.to_python(u7s)), str)
        uv = self.validator(outputEncoding='utf-7')
        self.assertEqual(uv.from_python(us), u7s)
        self.assertIs(type(uv.from_python(us)), bytes)
        uv = self.validator(inputEncoding=None)
        self.assertEqual(uv.to_python(us), us)
        self.assertIs(type(uv.to_python(us)), str)
        self.assertEqual(uv.from_python(us), u8s)
        self.assertIs(type(uv.from_python(us)), bytes)
        uv = self.validator(outputEncoding=None)
        self.assertEqual(uv.to_python(u8s), us)
        self.assertIs(type(uv.to_python(u8s)), str)
        self.assertEqual(uv.from_python(us), us)
        self.assertIs(type(uv.from_python(us)), str)

        def test_unicode_empty(self):
            iv = self.validator()
            for value in [None, b"", ""]:
                result = iv.to_python(value)
                self.assertEqual(result, "")
                self.assertIsInstance(result, str)


class TestIntValidator(unittest.TestCase):

    def setUp(self):
        self.validator = validators.Int
        self.messages = self.validator.message

    def test_docstring(self):
        doc = self.validator.__doc__
        self.assertIn('Please enter an integer value', doc)

    def test_int_min(self):
        iv = self.validator(min=5)
        self.assertEqual(iv.to_python("5"), 5)
        self.assertEqual(validate(iv, "1"),
            self.messages('tooLow', None, min=5))

    def test_int_max(self):
        iv = self.validator(max=10)
        self.assertEqual(iv.to_python("10"), 10)
        self.assertEqual(validate(iv, "15"),
            self.messages('tooHigh', None, max=10))

    def test_int_minmax_optional(self):
        iv = self.validator(min=5, max=10, if_empty=None)
        self.assertIsNone(iv.to_python(""))
        self.assertIsNone(iv.to_python(None))
        self.assertEqual(iv.to_python('7'), 7)
        self.assertEqual(validate(iv, "1"),
            self.messages('tooLow', None, min=5))
        self.assertEqual(validate(iv, "15"),
            self.messages('tooHigh', None, max=10))

    def test_int_minmax_mandatory(self):
        iv = validators.Int(min=5, max=10, not_empty=True)
        self.assertEqual(validate(iv, None),
            self.messages('empty', None))
        self.assertEqual(validate(iv, "1"),
            self.messages('tooLow', None, min=5))
        self.assertEqual(validate(iv, "15"),
            self.messages('tooHigh', None, max=10))


class TestNumberValidator(unittest.TestCase):
    def setUp(self):
        self.validator = validators.Number

    def test_inf(self):
        """Validate infinity if system supports it."""
        # Ability to convert to infinity depends on your C
        # Float library as well as the python version.
        try:
            inf = float('infinity')
        except ValueError:
            pytest.skip('skipping incompatible float test')
        self.assertEqual(self.validator.to_python('infinity'), inf)


class TestDateConverterValidator(unittest.TestCase):

    def setUp(self):
        self.validator = validators.DateConverter

    def test_bad_dates(self):
        dc = self.validator(month_style='dd/mm/yyyy')
        try:
            dc.to_python('20/12/150')
        except Invalid as e:
            self.assertIn(
                'Please enter a four-digit year after 1899', str(e))
        else:
            self.fail('Date should be invalid')
        try:
            dc.to_python('oh/happy/day')
        except Invalid as e:
            self.assertIn(
                'Please enter the date in the form DD/MM/YYYY', str(e))
        else:
            self.fail('Date should be invalid')

    def test_month_style_alias(self):
        dc = self.validator()
        self.assertEqual(dc.month_style, 'mdy')
        for style in 'mdy mm/dd/yyyy MM/DD/YYYY US American'.split():
            dc = self.validator(month_style=style)
            self.assertEqual(dc.month_style, 'mdy')
        for style in 'dmy dd/mm/yyyy DD/MM/YYYY Euro European'.split():
            dc = self.validator(month_style=style)
            self.assertEqual(dc.month_style, 'dmy')
        for style in 'ymd yyyy/mm/dd YYYY/MM/DD ISO China Chinese'.split():
            dc = self.validator(month_style=style)
            self.assertEqual(dc.month_style, 'ymd')
        try:
            self.validator(month_style='Klingon')
        except TypeError as e:
            self.assertEqual(str(e), "Bad month_style: 'klingon'")
        else:
            self.fail('month_style should be invalid')
        try:
            self.validator(month_style='ydm')
        except TypeError as e:
            self.assertEqual(
                str(e), "Bad month_style: 'ydm'")
        else:
            self.fail('month_style should be invalid')

    def test_us_style(self):
        d = datetime.date(2007, 12, 20)
        dc = self.validator()
        self.assertEqual(dc.to_python('12/20/2007'), d)
        self.assertEqual(dc.from_python(d), '12/20/2007')
        self.assertEqual(dc.to_python('Dec/20/2007'), d)
        self.assertEqual(dc.to_python('December/20/2007'), d)
        try:
            self.assertEqual(dc.to_python('20/12/2007'), d)
        except Invalid as e:
            self.assertIn('Please enter a month from 1 to 12', str(e))
        else:
            self.fail('Date should be invalid')
        try:
            self.assertEqual(dc.to_python('12/Dec/2007'), d)
        except Invalid as e:
            self.assertIn(
                'Please enter the date in the form MM/DD/YYYY', str(e))
        else:
            self.fail('Date should be invalid')

    def test_euro_style(self):
        d = datetime.date(2007, 12, 20)
        dc = self.validator(month_style='dd/mm/yyyy')
        self.assertEqual(dc.to_python('20/12/2007'), d)
        self.assertEqual(dc.from_python(d), '20/12/2007')
        self.assertEqual(dc.to_python('20/Dec/2007'), d)
        self.assertEqual(dc.to_python('20/December/2007'), d)
        try:
            self.assertEqual(dc.to_python('12/20/2007'), d)
        except Invalid as e:
            self.assertIn('Please enter a month from 1 to 12', str(e))
        else:
            self.fail('Date should be invalid')
        try:
            self.assertEqual(dc.to_python('Dec/12/2007'), d)
        except Invalid as e:
            self.assertIn(
                'Please enter the date in the form DD/MM/YYYY', str(e))
        else:
            self.fail('Date should be invalid')

    def test_iso_style(self):
        d = datetime.date(2013, 6, 30)
        dc = self.validator(month_style='yyyy/mm/dd')
        self.assertEqual(dc.to_python('2013/06/30'), d)
        self.assertEqual(dc.from_python(d), '2013/06/30')
        self.assertEqual(dc.to_python('2013/Jun/30'), d)
        self.assertEqual(dc.to_python('2013/June/30'), d)
        try:
            self.assertEqual(dc.to_python('2013/30/06'), d)
        except Invalid as e:
            self.assertIn('Please enter a month from 1 to 12', str(e))
        else:
            self.fail('Date should be invalid')
        try:
            self.assertEqual(dc.to_python('2013/06/Jun'), d)
        except Invalid as e:
            self.assertIn(
                'Please enter the date in the form YYYY/MM/DD', str(e))
        else:
            self.fail('Date should be invalid')

    def test_no_day(self):
        d = datetime.date(2007, 12, 1)
        dc = self.validator(accept_day=False)
        self.assertEqual(dc.to_python('12/2007'), d)
        self.assertEqual(dc.from_python(d), '12/2007')
        self.assertEqual(dc.to_python('Dec/2007'), d)
        self.assertEqual(dc.to_python('December/2007'), d)
        try:
            self.assertEqual(dc.to_python('20/2007'), d)
        except Invalid as e:
            self.assertIn('Please enter a month from 1 to 12', str(e))
        else:
            self.fail('Date should be invalid')
        try:
            self.assertEqual(dc.to_python('12/20/2007'), d)
        except Invalid as e:
            self.assertIn(
                'Please enter the date in the form MM/YYYY', str(e))
        else:
            self.fail('Date should be invalid')
        try:
            self.assertEqual(dc.to_python('2007/Dec'), d)
        except Invalid as e:
            self.assertIn(
                'Please enter the date in the form MM/YYYY', str(e))
        else:
            self.fail('Date should be invalid')

    def test_from_python(self):
        d = datetime.date(2013, 6, 30)
        dc = self.validator()
        self.assertEqual(dc.from_python(d), '06/30/2013')
        dc = self.validator(month_style='dd/mm/yyyy')
        self.assertEqual(dc.from_python(d), '30/06/2013')
        dc = self.validator(month_style='dd/mm/yyyy', separator='.')
        self.assertEqual(dc.from_python(d), '30.06.2013')
        dc = self.validator(month_style='yyyy/mm/dd')
        self.assertEqual(dc.from_python(d), '2013/06/30')
        dc = self.validator(month_style='yyyy/mm/dd', separator='-')
        self.assertEqual(dc.from_python(d), '2013-06-30')

    def test_from_python_no_day(self):
        d = datetime.date(2013, 6, 30)
        dc = self.validator(accept_day=False)
        self.assertEqual(dc.from_python(d), '06/2013')
        dc = self.validator(month_style='dd/mm/yyyy', accept_day=False)
        self.assertEqual(dc.from_python(d), '06/2013')
        dc = self.validator(month_style='yyyy/mm/dd', accept_day=False)
        self.assertEqual(dc.from_python(d), '2013/06')
        dc = self.validator(month_style='yyyy/mm/dd',
            separator='-', accept_day=False)
        self.assertEqual(dc.from_python(d), '2013-06')

    def test_auto_separator(self):
        d = datetime.date(2013, 6, 30)
        dc = self.validator(month_style='us', separator=None)
        self.assertEqual(dc.from_python(d), '06/30/2013')
        dc = self.validator(month_style='euro', separator=None)
        self.assertEqual(dc.from_python(d), '30.06.2013')
        dc = self.validator(month_style='iso', separator=None)
        self.assertEqual(dc.from_python(d), '2013-06-30')


class TestTimeConverterValidator(unittest.TestCase):

    def setUp(self):
        self.validator = validators.TimeConverter

    def test_time(self):
        tc = self.validator()
        self.assertEqual(tc.to_python('20:30:15'), (20, 30, 15))
        try:
            tc.to_python('25:30:15')
        except Invalid as e:
            self.assertIn(
                'You must enter an hour in the range 0-23', str(e))
        else:
            self.fail('Time should be invalid')
        try:
            tc.to_python('20:75:15')
        except Invalid as e:
            self.assertIn(
                'You must enter a minute in the range 0-59', str(e))
        else:
            self.fail('Time should be invalid')
        try:
            tc.to_python('20:30:75')
        except Invalid as e:
            self.assertIn(
                'You must enter a second in the range 0-59', str(e))
        else:
            self.fail('Time should be invalid')
        try:
            tc.to_python('20:30:zx')
        except Invalid as e:
            self.assertIn(
                'The second value you gave is not a number', str(e))
            self.assertIn('zx', str(e))
        else:
            self.fail('Time should be invalid')


class TestForEachValidator(unittest.TestCase):

    def test_foreach_if_missing(self):

        class SubSchema(Schema):
            age = validators.Int()
            name = validators.String(not_empty=True)

        class MySchema(Schema):
            pre_validators = [NestedVariables()]
            people = ForEach(SubSchema(), if_missing=NoDefault,
                messages={'missing': 'Please add a person'})

        validator = MySchema()

        self.assertFalse(validator.fields['people'].not_empty)

        class State:
            pass

        start_values = {}
        state = State()

        try:
            values = validator.to_python(start_values, state)
        except Invalid as e:
            self.assertEqual(e.unpack_errors(),
                {'people': 'Please add a person'})
        else:
            raise Exception("Shouldn't be valid data", values, start_values)


class TestXRIValidator(unittest.TestCase):
    """Generic tests for the XRI validator"""

    def setUp(self):
        self.validator = validators.XRI

    def test_creation_valid_params(self):
        """The creation of an XRI validator with valid parameters must
        succeed"""
        self.validator()
        self.validator(True, "i-name")
        self.validator(True, "i-number")
        self.validator(False, "i-name")
        self.validator(False, "i-number")

    def test_creation_invalid_xri(self):
        """Only "i-name" and "i-number" are valid XRIs"""
        self.assertRaises(AssertionError, self.validator, True, 'i-something')

    def test_valid_simple_individual_iname_without_type(self):
        """XRIs must start with either an equals sign or an at sign"""
        validator = self.validator(True, "i-name")
        self.assertRaises(Invalid, validator.to_python, 'Gustavo')

    def test_valid_iname_with_schema(self):
        """XRIs may have their schema in the beggining"""
        validator = self.validator()
        self.assertEqual(validator.to_python('xri://=Gustavo'),
                         'xri://=Gustavo')

    def test_schema_is_added_if_asked(self):
        """The schema must be added to an XRI if explicitly asked"""
        validator = self.validator(True)
        self.assertEqual(validator.to_python('=Gustavo'),
                         'xri://=Gustavo')

    def test_schema_not_added_if_not_asked(self):
        """The schema must not be added to an XRI unless explicitly asked"""
        validator = self.validator()
        self.assertEqual(validator.to_python('=Gustavo'), '=Gustavo')

    def test_spaces_are_trimmed(self):
        """Spaces at the beggining or end of the XRI are removed"""
        validator = self.validator()
        self.assertEqual(validator.to_python('    =Gustavo  '), '=Gustavo')


class TestINameValidator(unittest.TestCase):
    """Tests for the XRI i-names validator"""

    def setUp(self):
        self.validator = validators.XRI(xri_type="i-name")

    def test_valid_global_individual_iname(self):
        """Global & valid individual i-names must pass validation"""
        self.validator.to_python('=Gustavo')

    def test_valid_global_organizational_iname(self):
        """Global & valid organizational i-names must pass validation"""
        self.validator.to_python('@Canonical')

    def test_invalid_iname(self):
        """Non-string i-names are rejected"""
        self.assertRaises(Invalid, self.validator._validate_python, None)

    def test_exclamation_in_inames(self):
        """Exclamation marks at the beggining of XRIs is something specific
        to i-numbers and must be rejected in i-names"""
        self.assertRaises(Invalid, self.validator.to_python,
                          "!!1000!de21.4536.2cb2.8074")

    def test_repeated_characters(self):
        """Dots and dashes must not be consecutively repeated in i-names"""
        self.assertRaises(Invalid, self.validator.to_python,
                          "=Gustavo--Narea")
        self.assertRaises(Invalid, self.validator.to_python,
                          "=Gustavo..Narea")

    def test_punctuation_marks_at_beggining(self):
        """Punctuation marks at the beggining of i-names are forbidden"""
        self.assertRaises(Invalid, self.validator.to_python,
                          "=.Gustavo")
        self.assertRaises(Invalid, self.validator.to_python,
                          "=-Gustavo.Narea")

    def test_numerals_at_beggining(self):
        """Numerals at the beggining of i-names are forbidden"""
        self.assertRaises(Invalid, self.validator.to_python,
                          "=1Gustavo")

    def test_non_english_inames(self):
        """i-names with non-English characters are valid"""
        self.validator.to_python('=Gustavo.Narea.García')
        self.validator.to_python('@名前.例')

    def test_inames_plus_paths(self):
        """i-names with paths are valid but not supported"""
        self.assertRaises(Invalid, self.validator.to_python,
                          "=Gustavo/(+email)")

    def test_communities(self):
        """i-names may have so-called 'communities'"""
        self.validator.to_python('=María*Yolanda*Liliana*Gustavo')
        self.validator.to_python('=Gustavo*Andreina')
        self.validator.to_python('@IBM*Lenovo')


class TestINumberValidator(unittest.TestCase):
    """Tests for the XRI i-number validator"""

    def setUp(self):
        self.validator = validators.XRI(xri_type="i-number")

    def test_valid_global_personal_inumber(self):
        """Global & valid personal i-numbers must pass validation"""
        self.validator.to_python('=!1000.a1b2.93d2.8c73')

    def test_valid_global_organizational_inumber(self):
        """Global & valid organizational i-numbers must pass validation"""
        self.validator.to_python('@!1000.a1b2.93d2.8c73')

    def test_valid_global_network_inumber(self):
        """Global & valid network i-numbers must pass validation"""
        self.validator.to_python('!!1000')

    def test_valid_community_personal_inumbers(self):
        """Community & valid personal i-numbers must pass validation"""
        self.validator.to_python('=!1000.a1b2.93d2.8c73!3ae2!1490')

    def test_valid_community_organizational_inumber(self):
        """Community & valid organizational i-numbers must pass validation"""
        self.validator.to_python('@!1000.9554.fabd.129c!2847.df3c')

    def test_valid_community_network_inumber(self):
        """Community & valid network i-numbers must pass validation"""
        self.validator.to_python('!!1000!de21.4536.2cb2.8074')


class TestOpenIdValidator(unittest.TestCase):
    """Tests for the OpenId validator"""

    def setUp(self):
        self.validator = validators.OpenId(add_schema=False)

    def test_url(self):
        self.assertEqual(self.validator.to_python('http://example.org'),
                         'http://example.org')

    def test_iname(self):
        self.assertEqual(self.validator.to_python('=Gustavo'), '=Gustavo')

    def test_inumber(self):
        self.assertEqual(self.validator.to_python('!!1000'), '!!1000')

    def test_email(self):
        """Email addresses are not valid OpenIds!"""
        self.assertRaises(Invalid, self.validator.to_python,
                          "wait@busstop.com")

    def test_prepending_schema(self):
        validator = validators.OpenId(add_schema=True)
        self.assertEqual(validator.to_python("example.org"),
                         "http://example.org")
        self.assertEqual(validator.to_python("=Gustavo"),
                         "xri://=Gustavo")
        self.assertEqual(validator.to_python("!!1000"),
                         "xri://!!1000")


class TestOneOfValidator(unittest.TestCase):

    def test_docstring(self):
        doc = validators.OneOf.__doc__
        self.assertIn(
            'Value must be one of: ``%(items)s`` (not ``%(value)r``)', doc)

    def test_unicode_list(self):
        o = validators.OneOf(['ö', 'a'])
        self.assertRaises(Invalid, o.to_python, "ä")

    def test_ascii_list(self):
        o = validators.OneOf(['a', 'b'])
        self.assertRaises(Invalid, o.to_python, 'c')

    def test_int_list_list(self):

        class foo(Schema):
            bar = validators.OneOf((1, 2, 3), testValueList=True)

        expected = {'bar': (1, 2, 3)}
        value = foo.to_python(expected)

        self.assertEqual(expected, value)

    def test_mixed_list(self):
        o = validators.OneOf(['a', 'b', 'ö', '™', 1, 2, 3])
        try:
            o.to_python('c')
            raise ValueError("exception expected")
        except Invalid as e:
            self.assertIn("Value must be one of: a; b; ö; ™; 1; 2; 3 (not 'c')", str(e))
        try:
            o.to_python(9)
            raise ValueError("exception expected")
        except Invalid as e:
            self.assertIn("Value must be one of: a; b; ö; ™; 1; 2; 3 (not 9)", str(e))
        try:
            o.to_python('d')
            raise ValueError("exception expected")
        except Invalid as e:
            self.assertIn("Value must be one of: a; b; ö; ™; 1; 2; 3 (not 'd')", str(e))
        try:
            o.to_python('§')
            raise ValueError("exception expected")
        except Invalid as e:
            self.assertIn("Value must be one of: a; b; ö; ™; 1; 2; 3 (not '§')", str(e))


class TestIPAddressValidator(unittest.TestCase):

    def setUp(self):
        self.validator = validators.IPAddress

    def test_valid_address(self):
        self.validator().to_python('127.0.0.1')

    def test_address_is_none(self):
        self.assertRaises(Invalid, self.validator()._validate_python, None)

    def test_invalid_address(self):
        validate = self.validator().to_python
        self.assertRaises(Invalid, validate, '127.0.1')
        self.assertRaises(Invalid, validate, '271.0.0.1')
        self.assertRaises(Invalid, validate, '127.0.0.0.1')

    def test_leading_zeros(self):
        validate = self.validator().to_python
        try:
            validate('1.2.3.037')
        except Invalid as e:
            self.assertIn('The octets must not have leading zeros', str(e))
        else:
            self.fail('IP address octets with leading zeros should be invalid')
        try:
            validate('1.2.3.0377')
        except Invalid as e:
            self.assertIn('The octets must not have leading zeros', str(e))
        else:
            self.fail('IP octets with leading zeros should be invalid')

    def test_leading_zeros_allowed(self):
        validate = self.validator(leading_zeros=True).to_python
        try:
            validate('1.2.3.037')
        except Invalid:
            self.fail('IP address octets with leading zeros should be valid')
        try:
            validate('1.2.3.0377')
        except Invalid as e:
            self.assertIn(
                "The octets must be within the range of 0-255 (not '377')",
                str(e))
        else:
            self.fail(
                'IP address octets should not be interpreted as octal numbers')


class TestURLValidator(unittest.TestCase):

    def setUp(self):
        self.validator = validators.URL()

    def test_cojp(self):
        self.assertEqual(
            self.validator.to_python('http://domain.co.jp'),
            'http://domain.co.jp')

    def test_1char_thirdlevel(self):
        self.assertEqual(self.validator.to_python(
            'http://c.somewhere.pl/wi16677/5050f81b001f9e5f45902c1b/'),
            'http://c.somewhere.pl/wi16677/5050f81b001f9e5f45902c1b/')

    def test_ip_validator(self):
        self.assertEqual(validators.URL(require_tld=False).to_python(
            "http://65.18.195.155/cgi-ordoro/bo/start.cgi"),
            "http://65.18.195.155/cgi-ordoro/bo/start.cgi")

    def test_invalid_url(self):
        self.assertRaises(validators.Invalid, self.validator.to_python,
            '[http://domain.co.jp')


class TestRequireIfMissingValidator(unittest.TestCase):

    def setUp(self):
        self.validator = validators.RequireIfMissing

    def test_missing(self):
        v = self.validator('phone_type', missing='mail')
        self.assertEqual(
            validate(v, dict(phone_type='')),
            dict(phone_type='Please enter a value'))
        self.assertEqual(
            validate(v, dict(phone_type='', mail='foo@bar.org')),
            dict(phone_type='', mail='foo@bar.org'))

    def test_present(self):
        v = self.validator('phone_type', present='phone')
        self.assertEqual(
            validate(v, dict(phone_type='', phone='510 420  4577')),
            dict(phone_type='Please enter a value'))
        self.assertEqual(
            validate(v, dict(phone='')), dict(phone=''))

    def test_zero(self):
        v = self.validator('operator', present='operand')
        self.assertEqual(
            validate(v, dict(operator='', operand=0)),
            dict(operator='Please enter a value'))


class TestRequireIfMatchingValidator(unittest.TestCase):

    def setUp(self):
        self.validator = validators.RequireIfMatching

    def test_missing(self):
        v = self.validator('phone_type', 'mobile', required_fields=['mobile'])
        self.assertEqual(
            validate(v, dict()),
            dict()
        )

    def test_matching(self):
        v = self.validator('phone_type', 'mobile', required_fields=['mobile'])
        self.assertEqual(
            validate(v, dict(phone_type='mobile')),
            dict(mobile='Please enter a value')
        )

    def test_matching_multiple_required(self):
        v = self.validator('phone_type', 'mobile', required_fields=['mobile', 'code'])
        self.assertEqual(
            validate(v, dict(phone_type='mobile')),
            dict(mobile='Please enter a value')
        )

    def test_not_matching(self):
        v = self.validator('phone_type', 'home', required_fields=['mobile'])
        self.assertEqual(
            validate(v, dict(phone_type='mobile')),
            dict(phone_type='mobile')
        )


class TestCondfirmType(unittest.TestCase):

    def test_schema_dict(self):
        class foo(Schema):
            bar = validators.ConfirmType(subclass=(dict,))

        validator = foo()
        value = {'bar': {}}
        self.assertEqual(value, validator.to_python(value))

        self.assertRaises(validators.Invalid, validator.to_python, ({'bar': 1},))

    def test_schema_int(self):
        class foo(Schema):
            bar = validators.ConfirmType(subclass=(int,))

        validator = foo()
        value = {'bar': 1}
        self.assertEqual(value, validator.to_python(value))

        self.assertRaises(validators.Invalid, validator.to_python, ({'bar': {}},))



"""FormEncode validates and converts nested structures.

It allows for a declarative form of defining the validation,
and decoupled processes for filling and generating forms.

The official repo is at GitHub: https://github.com/formencode/formencode
"""
from setuptools import setup

setup()



import unittest

from urllib.parse import parse_qsl

from formencode import Invalid, Validator, compound, foreach, validators
from formencode.schema import Schema, merge_dicts, SimpleFormValidator
from formencode.variabledecode import NestedVariables

import pytest


def _notranslation(s):
    return s


def setup_module(module):
    """Disable i18n translation"""

    import builtins
    builtins._ = _notranslation


def teardown_module(module):
    """Remove translation function"""
    import builtins
    del builtins._


def d(**kw):
    return kw


def cgi_parse(qs):
    """Parse a query string and returns the usually dictionary."""
    d = {}
    for key, value in parse_qsl(qs, True):
        if key in d:
            if isinstance(d[key], list):
                d[key].append(value)
            else:
                d[key] = [d[key], value]
        else:
            d[key] = value
    return d


class DecodeCase:

    error_expected = False

    def __init__(self, schema, input, **output):
        if isinstance(input, str):
            input = cgi_parse(input)
        self.input = input
        self.schema = schema
        self.output = output
        all_cases.append(self)

    def test(self):
        print('input', repr(self.input))
        actual = self.schema.to_python(self.input)
        print('output', repr(actual))
        assert actual == self.output


class BadCase(DecodeCase):

    error_expected = True

    def __init__(self, *args, **kw):
        DecodeCase.__init__(self, *args, **kw)
        if len(self.output) == 1 and 'text' in self.output:
            self.output = self.output['text']

    def test(self):
        print(repr(self.input))
        try:
            print(repr(self.schema.to_python(self.input)))
        except Invalid as e:
            actual = e.unpack_errors()
            assert actual == self.output
        else:
            assert False, "Exception expected"


class Name(Schema):
    fname = validators.String(not_empty=True)
    mi = validators.String(max=1, if_missing=None, if_empty=None)
    lname = validators.String(not_empty=True)


all_cases = []

DecodeCase(Name, 'fname=Ian&mi=S&lname=Bicking',
           fname='Ian', mi='S', lname='Bicking')

DecodeCase(Name, 'fname=Ian&lname=Bicking',
           fname='Ian', mi=None, lname='Bicking')

BadCase(Name, 'fname=&lname=',
        fname='Please enter a value',
        lname='Please enter a value')

BadCase(Name, 'fname=Franklin&mi=Delano&lname=Roosevelt',
        mi="Enter a value not more than 1 characters long")

BadCase(Name, '',
        fname='Missing value',
        lname='Missing value')


class AddressesForm(Schema):

    pre_validators = [NestedVariables()]

    class addresses(foreach.ForEach):

        class schema(Schema):
            name = Name()
            email = validators.Email()


DecodeCase(AddressesForm,
           'addresses-2.name.fname=Jill&addresses-1.name.fname=Bob&'
           'addresses-1.name.lname=Briscoe&'
           'addresses-1.email=bob@bobcom.com&'
           'addresses-2.name.lname=Hill&addresses-2.email=jill@hill.com&'
           'addresses-2.name.mi=J',
           addresses=[d(name=d(fname='Bob', mi=None, lname='Briscoe'),
                        email='bob@bobcom.com'),
                      d(name=d(fname='Jill', mi='J', lname='Hill'),
                        email='jill@hill.com')])

DecodeCase(AddressesForm,
           '',
           addresses=[])

BadCase(AddressesForm,
        'addresses-1.name.fname=&addresses-1.name.lname=x&'
        'addresses-1.email=x@domain.com',
        addresses=[d(name=d(fname="Please enter a value"))])

BadCase(AddressesForm,
        'whatever=nothing',
        text="The input field 'whatever' was not expected.")


def collect_cases():
    for case in all_cases:
        yield case.test


@pytest.mark.parametrize('func', list(collect_cases()))
def test_this(func):
    func()


def test_merge():
    assert (merge_dicts(dict(a='a'), dict(b='b'))
            == dict(a='a', b='b'))
    assert (merge_dicts(dict(a='a', c='c'), dict(a='a', b='b'))
            == dict(a='a\na', b='b', c='c'))
    assert (merge_dicts(dict(a=['a1', 'a2'], b=['b'], c=['c']),
                        dict(a=['aa1'],
                             b=['bb', 'bbb'],
                             c='foo'))
            == dict(a=['a1\naa1', 'a2'], b=['b\nbb', 'bbb'],
                    c=['c']))


class ChainedTest(Schema):
    a = validators.String()
    a_confirm = validators.String()

    b = validators.String()
    b_confirm = validators.String()

    chained_validators = [
        validators.FieldsMatch('a', 'a_confirm'),
        validators.FieldsMatch('b', 'b_confirm')]


def test_multiple_chained_validators_errors():
    s = ChainedTest()
    try:
        s.to_python({'a': '1', 'a_confirm': '2', 'b': '3', 'b_confirm': '4'})
    except Invalid as e:
        assert 'a_confirm' in e.error_dict and 'b_confirm' in e.error_dict
    try:
        s.to_python({})
    except Invalid:
        pass
    else:
        assert False


def test_SimpleFormValidator_doc():
    """
    Verify SimpleFormValidator preserves the decorated function's docstring.
    """

    BOGUS_DOCSTRING = "blah blah blah"

    def f(value_dict, state, validator):
        value_dict['f'] = 99

    f.__doc__ = BOGUS_DOCSTRING
    g = SimpleFormValidator(f)

    assert f.__doc__ == g.__doc__, "Docstrings don't match!"


class State:
    pass


def test_state_manipulation():
    """
    Verify that full_dict push and pop works
    """
    state = State()
    old_dict = state.full_dict = {'a': 1}
    old_key = state.key = 'a'
    new_dict = {'b': 2}

    class MyValidator(Validator):
        check_key = None
        pre_validator = False
        post_validator = False
        __unpackargs__ = ('check_key',)

        def to_python(self, value, state):
            if not self.pre_validator:
                assert getattr(
                    state, 'full_dict', {}) == new_dict, "full_dict not added"
            assert state.key == self.check_key, "key not updated"

            return value

        def from_python(self, value, state):
            if not self.post_validator:
                assert getattr(
                    state, 'full_dict', {}) == new_dict, "full_dict not added"
            assert state.key == self.check_key, "key not updated"

            return value

    s = Schema(if_key_missing=None, b=MyValidator('b'), c=MyValidator('c'),
               pre_validators=[MyValidator('a', pre_validator=True)],
               chained_validators=[MyValidator('a', post_validator=True)])

    s.to_python(new_dict, state)

    assert state.full_dict == old_dict, "full_dict not restored"
    assert state.key == old_key, "key not restored"

    s.from_python(new_dict, state)

    assert state.full_dict == old_dict, "full_dict not restored"
    assert state.key == old_key, "key not restored"


class TestAtLeastOneCheckboxIsChecked(unittest.TestCase):
    """Tests to address SourceForge bug #1777245

    The reporter is trying to enforce agreement to a Terms of Service
    agreement, with failure to check the 'I agree' checkbox handled as
    a validation failure. The tests below illustrate a working approach.
    """

    def setUp(self):
        self.not_empty_messages = {'missing': 'a missing value message'}

        class CheckForCheckboxSchema(Schema):
            agree = validators.StringBool(messages=self.not_empty_messages)

        self.schema = CheckForCheckboxSchema()

    def test_Schema_with_input_present(self):
        # <input type="checkbox" name="agree" value="yes" checked="checked">
        result = self.schema.to_python({'agree': 'yes'})
        self.assertTrue(result['agree'])

    def test_Schema_with_input_missing(self):
        # <input type="checkbox" name="agree" value="yes">
        try:
            self.schema.to_python({})
        except Invalid as exc:
            error_message = exc.error_dict['agree'].msg
            self.assertEqual(self.not_empty_messages['missing'], error_message,
                error_message)
        else:
            self.fail('missing input not detected')


class TestStrictSchemaWithMultipleEqualInputFields(unittest.TestCase):
    """Tests to address github bug #13"""

    def setUp(self):

        class StrictSchema(Schema):
            allow_extra_fields = False

        class IntegerTestSchema(StrictSchema):
            field = validators.Int(not_empty=True)

        class StringTestSchema(StrictSchema):
            field = validators.UnicodeString(not_empty=True)

        class CorrectForEachStringTestSchema(StrictSchema):
            field = foreach.ForEach(validators.UnicodeString(not_empty=True))

        class CorrectSetTestSchema(StrictSchema):
            field = validators.Set(not_empty=True)

        class CorrectSetTestPipeSchema(StrictSchema):
            field = compound.Pipe(validators.Set(not_empty=True),
                foreach.ForEach(validators.UnicodeString(not_empty=True)))

        self.int_schema = IntegerTestSchema()
        self.string_schema = StringTestSchema()
        self.foreach_schema = CorrectForEachStringTestSchema()
        self.set_schema = CorrectSetTestSchema()
        self.pipe_schema = CorrectSetTestPipeSchema()

    def test_single_integer_value(self):
        params = cgi_parse('field=111')
        data = self.int_schema.to_python(params)
        self.assertEqual(data, dict(field=111))

    def test_multiple_integer_value(self):
        params = cgi_parse('field=111&field=222')
        self.assertRaises(Invalid, self.int_schema.to_python, params)

    def test_single_string_value(self):
        params = cgi_parse('field=string')
        data = self.string_schema.to_python(params)
        self.assertEqual(data, dict(field='string'))

    def test_multiple_string_value(self):
        params = cgi_parse('field=string1&field=string2')
        self.assertRaises(Invalid, self.string_schema.to_python, params)

    def test_correct_multiple_string_value_foreach(self):
        params = cgi_parse('field=string1&field=string2')
        data = self.foreach_schema.to_python(params)
        self.assertEqual(data, dict(field=['string1', 'string2']))

    def test_correct_multiple_string_value_set(self):
        params = cgi_parse('field=string1&field=string2')
        data = self.set_schema.to_python(params)
        self.assertEqual(data, dict(field=['string1', 'string2']))

    def test_correct_multiple_string_value_pipe(self):
        params = cgi_parse('field=string1&field=string2')
        data = self.pipe_schema.to_python(params)
        self.assertEqual(data, dict(field=['string1', 'string2']))


def test_copy():
    assert 'mi' in Name.fields
    NoTitleName = Name()
    assert 'mi' in NoTitleName.fields
    TitleName = NoTitleName(title=validators.String())
    assert 'mi' in TitleName.fields
    assert 'title' in TitleName.fields
    assert 'title' not in NoTitleName.fields



import formencode

ne = formencode.validators.NotEmpty()


def _test_builtins(func):
    def dummy(s):
        return "builtins dummy"
    import builtins
    builtins._ = dummy

    try:
        ne.to_python("")
    except formencode.api.Invalid as e:
        func(e)

    del builtins._


def test_builtins():
    def withbuiltins(e):
        assert str(e) == "builtins dummy"

    _test_builtins(withbuiltins)


def test_bultins_disabled():
    def withoutbuiltins(e):
        assert str(e) != "builtins dummy"

    ne.use_builtins_gettext = False
    _test_builtins(withoutbuiltins)


def test_state():
    class st:
        def _(self, s):
            return "state dummy"

    try:
        ne.to_python("", state=st())
    except formencode.api.Invalid as e:
        assert str(e) == "state dummy"


def _test_lang(language, notemptytext):

    formencode.api.set_stdtranslation(languages=[language])

    try:
        ne.to_python("")
    except formencode.api.Invalid as e:
        assert str(e) == notemptytext

    formencode.api.set_stdtranslation()  # set back to defaults


def test_de():
    _test_lang("de", "Bitte einen Wert eingeben")


def test_es():
    _test_lang("es", "Por favor introduzca un valor")


def test_pt_BR():
    _test_lang("pt_BR", "Por favor digite um valor")


def test_zh_TW():
    _test_lang("zh_TW", "請輸入值")


def test_sk():
    _test_lang("sk", "Zadajte hodnotu, prosím")


def test_ru():
    _test_lang("ru", "Необходимо ввести значение")


def test_sl():
    _test_lang("sl", "Prosim, izpolnite polje")


def test_pt_PT():
    _test_lang("pt_PT", "Por favor insira um valor")


def test_fr():
    _test_lang("fr", "Saisissez une valeur")


def test_nl():
    _test_lang("nl", "Voer een waarde in")


def test_pl():
    _test_lang("pl", "Proszę podać wartość")


def test_el():
    _test_lang("el", "Παρακαλούμε εισάγετε μια τιμή")


def test_zh_CN():
    _test_lang("zh_CN", "请输入一个值")


def test_cs():
    _test_lang("cs", "Prosím zadejte hodnotu")


def test_fi():
    _test_lang("fi", "Anna arvo")


def test_nb_NO():
    _test_lang("nb_NO", "Venligst fyll inn en verdi")


def test_it():
    _test_lang("it", "Inserire un dato")


def test_et():
    _test_lang("et", "Palun sisestada väärtus")


def test_lt():
    _test_lang("lt", "Prašome įvesti reikšmę")


def test_ja():
    _test_lang("ja", "入力してください")


def test_tr():
    _test_lang("tr", "Lütfen bir değer giriniz")
